"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DcqlPresentationQuery = void 0;
const v = __importStar(require("valibot"));
const m_dcql_query_result_js_1 = require("../dcql-query-result/m-dcql-query-result.js");
const dcql_query_js_1 = require("../dcql-query/dcql-query.js");
const e_dcql_js_1 = require("../e-dcql.js");
const u_dcql_js_1 = require("../u-dcql.js");
var DcqlPresentationQuery;
(function (DcqlPresentationQuery) {
    DcqlPresentationQuery.vModel = v.object({
        ...m_dcql_query_result_js_1.DcqlQueryResult.vModel.entries,
        canBeSatisfied: v.literal(true),
        presentation_matches: v.record(v.pipe(v.string(), v.regex(u_dcql_js_1.idRegex)), v.object({
            ...v.omit(m_dcql_query_result_js_1.DcqlQueryResult.vModel.entries.credential_matches.value.options[0], ['all', 'issues', 'credential_index']).entries,
            presentation_index: v.number(),
        })),
    });
    DcqlPresentationQuery.parse = (input) => {
        return v.parse(DcqlPresentationQuery.vModel, input);
    };
    /**
     * Query if the presentation record can be satisfied by the provided presentations
     * considering the dcql query
     *
     * @param dcqlQuery
     * @param presentations
     */
    DcqlPresentationQuery.query = (presentations, ctx) => {
        const { dcqlQuery } = ctx;
        const result = (0, dcql_query_js_1.performDcqlQuery)(dcqlQuery, {
            credentials: presentations,
            presentation: true,
        });
        if (!result.canBeSatisfied) {
            return { ...result, canBeSatisfied: false };
        }
        return DcqlPresentationQuery.fromDcqlQueryResult(result);
    };
    DcqlPresentationQuery.validate = (dcqlQueryResult) => {
        if (!dcqlQueryResult.canBeSatisfied) {
            throw new e_dcql_js_1.DcqlInvalidPresentationRecordError({
                message: 'Invalid Presentation record',
                cause: dcqlQueryResult,
            });
        }
        return dcqlQueryResult;
    };
    DcqlPresentationQuery.fromDcqlQueryResult = (dcqlQueryResult) => {
        const { canBeSatisfied } = dcqlQueryResult;
        if (!canBeSatisfied) {
            throw new e_dcql_js_1.DcqlInvalidPresentationRecordError({
                message: 'Invalid Presentation record',
                cause: dcqlQueryResult,
            });
        }
        const presentation_matches = {};
        if (!dcqlQueryResult.credential_sets) {
            for (const credentialQueryId of dcqlQueryResult.credentials.map(c => c.id)) {
                const match = dcqlQueryResult.credential_matches[credentialQueryId];
                if (!match?.success) {
                    throw new e_dcql_js_1.DcqlInvalidPresentationRecordError({
                        message: `Credential query ${credentialQueryId} is required but not satisfied.`,
                    });
                }
                const { all, issues, credential_index, ...rest } = match;
                presentation_matches[credentialQueryId] = {
                    ...rest,
                    presentation_index: credential_index,
                };
            }
            return {
                ...dcqlQueryResult,
                canBeSatisfied,
                presentation_matches,
            };
        }
        for (const credentialSet of dcqlQueryResult.credential_sets ?? []) {
            const matchingOption = credentialSet.matching_options?.find(Boolean);
            if (!matchingOption)
                throw new Error('Invalid matching option');
            for (const credentialQueryId of matchingOption) {
                const match = dcqlQueryResult.credential_matches[credentialQueryId];
                if (match?.success) {
                    const { all, issues, credential_index, ...rest } = match;
                    presentation_matches[credentialQueryId] = {
                        ...rest,
                        presentation_index: credential_index,
                    };
                    continue;
                }
                if (credentialSet.required) {
                    throw new e_dcql_js_1.DcqlInvalidPresentationRecordError({
                        message: `Credential query ${credentialQueryId} is required but not satisfied.`,
                    });
                }
            }
        }
        return {
            ...dcqlQueryResult,
            canBeSatisfied,
            presentation_matches,
        };
    };
})(DcqlPresentationQuery || (exports.DcqlPresentationQuery = DcqlPresentationQuery = {}));
//# sourceMappingURL=m-dcql-presentation-query.js.map